use std::collections::{HashMap, HashSet};

use std::fs;

pub fn solve() {
    let path = "res/08/input";

    let contents = fs::read_to_string(path).expect("Something went wrong reading the file");

    let mut antennas: HashMap<char, Vec<(isize, isize)>> = HashMap::new();

    let rows = contents.lines().count();
    let cols = contents.lines().next().unwrap().chars().count();

    for (i, line) in contents.lines().enumerate() {
        for (j, c) in line.chars().enumerate() {
            match c {
                '.' => {}
                _ => {
                    let entry = antennas.entry(c).or_insert(Vec::new());
                    entry.push((i as isize, j as isize));
                }
            }
        }
    }

    let mut antinodes: HashSet<(isize, isize)> = HashSet::new();

    antennas.iter().for_each(|(_, v)| {
        v.iter().enumerate().for_each(|(i, (x1, y1))| {
            v.iter().skip(i + 1).for_each(|(x2, y2)| {
                if x1 == x2 && y1 == y2 {
                    return;
                }
                let (a_x1, a_y1) = (x1 + (x1 - x2), y1 + (y1 - y2));
                let (a_x2, a_y2) = (x2 + (x2 - x1), y2 + (y2 - y1));

                if a_x1 >= 0 && a_y1 >= 0 && a_x1 < rows as isize && a_y1 < cols as isize {
                    antinodes.insert((a_x1, a_y1));
                }
                if a_x2 >= 0 && a_y2 >= 0 && a_x2 < rows as isize && a_y2 < cols as isize {
                    antinodes.insert((a_x2, a_y2));
                }
            })
        })
    });

    let result = antinodes.len();

    println!("Result 1: {}", result);

    let mut antinodes: HashSet<(isize, isize)> = HashSet::new();

    antennas.iter().for_each(|(_, v)| {
        v.iter().enumerate().for_each(|(i, (x1, y1))| {
            v.iter().skip(i + 1).for_each(|(x2, y2)| {
                if x1 == x2 && y1 == y2 {
                    return;
                }

                let ax1_diff = x1 - x2;
                let ay1_diff = y1 - y2;
                let a1_gcd = gcd(ax1_diff, ay1_diff);

                let mut cnt = 0;
                loop {
                    let (ax, ay) = (x1 + ax1_diff * cnt / a1_gcd, y1 + ay1_diff * cnt / a1_gcd);
                    if ax < 0 || ay < 0 || ax >= rows as isize || ay >= cols as isize {
                        break;
                    }
                    antinodes.insert((ax, ay));
                    cnt += 1;
                }

                let ax2_diff = x2 - x1;
                let ay2_diff = y2 - y1;
                let a2_gcd = gcd(ax2_diff, ay2_diff);

                cnt = 0;
                loop {
                    let (ax, ay) = (x2 + ax2_diff * cnt / a2_gcd, y2 + ay2_diff * cnt / a2_gcd);
                    if ax < 0 || ay < 0 || ax >= rows as isize || ay >= cols as isize {
                        break;
                    }
                    antinodes.insert((ax, ay));
                    cnt += 1;
                }
            })
        })
    });

    let result = antinodes.len();

    println!("Result 2: {}", result);
}

fn gcd(mut a: isize, mut b: isize) -> isize {
    a = a.abs();
    b = b.abs();
    while b != 0 {
        let temp = b;
        b = a % b;
        a = temp;
    }
    a
}
